# frozen_string_literal: true

class CallbacksController < Devise::OmniauthCallbacksController
  skip_before_action :verify_authenticity_token

  def orcid
    authenticate_and_authorize('orcid')
  end

  def saml
    authenticate_and_authorize('saml')
  end

  def failure
    set_flash_message! :alert, :failure, kind: OmniAuth::Utils.camelize(failed_strategy.name), reason: failure_message
    redirect_to after_omniauth_failure_path_for(resource_name)
  end

  private

  def authenticate_and_authorize(provider)
    if current_user
      if current_user.orcid.nil? && provider.to_s == 'orcid'
        current_user.update(orcid: request.env["omniauth.auth"].uid)
        flash[:notice] = "You are now connected with ORCID"
      end
      redirect_to Hyrax::Engine.routes.url_helpers.dashboard_path
    else
      case provider.to_s
      when 'orcid'
        user = User.from_omniauth_orcid(request.env["omniauth.auth"])
        error_text = <<~TEXT
                      Login denied. It seems that your ORCID was not registered with RDMS. The RDMS service provider
                      will create an account for non-staff members of Ruhr-University Bochum of upon request by a patron
                      that itself must be a staff member of Ruhr-University Bochum.
        TEXT
      when 'saml'
        user = User.from_omniauth_saml(request.env["omniauth.auth"])
        error_text = <<~TEXT
                      Login denied. It seems you are not a staff member of Ruhr-University Bochum (Mitarbeiterstatus).
                      You can check your status role yourself in via RUB Identity-Management self service
                      (<a href="https://www.rub.de/login" target="_blank">https://www.rub.de/login</a>) or contacting
                      <a href="mailto:helpdesk@rub.de">helpdesk@rub.de</a> .  Users without staff status (Mitarbeiterstatus)
                      must use a different login option such as ORCID. To do so go back to the
                      <a href="/users/sign_in">login page</a>.
        TEXT
      else
        user = nil
        error_text = "Cannot authorize user. Omniauth provider not supported."
      end

      if user
        cookies[:login_type] = provider
        flash[:notice] = "You are now signed in as #{user.name} "
        sign_in_and_redirect user, event: :authentication
      else
        flash[:alert] = error_text
        redirect_to Hyrax::Engine.routes.url_helpers.contact_path
      end
    end
  end
end
