# frozen_string_literal: true

module Hyrax
  module CrcDatasets
    class ComplexSubjectsController < ApplicationController
      include HyraxHelper
      include ComplexHelper
      before_action :set_crc_dataset
      before_action :set_complex_subject, only: %i[show edit update]
      before_action :check_tombstone, except: %i[validate_subject_title]
      before_action :authorize
      with_themed_layout :decide_layout

      def new
        @complex_subject = ::ComplexSubject.new
      end

      def edit; end

      def show
        @subject_members = Kaminari.paginate_array(@complex_subject.file_sets_fast_load_metadata + ComplexSession.sort_by_title(@complex_subject.source_identifier)).page(params[:page] || 1).per(params[:per_page] || 10)
        @workflow = WorkflowPresenter.new(@crc_dataset, current_ability)
      end

      def create
        @complex_subject = ::ComplexSubject.new(complex_subject_params.merge!({ parent_source_identifier: @crc_dataset.source.first }))

        if @complex_subject.save
          update_file_set
          message = 'Complex subject was successfully created.'
          flash[:alert] = save_alert if params[:uploaded_files].present?
          redirect_to crc_dataset_complex_subject_path(@crc_dataset, @complex_subject), notice: message
        else
          render :new
        end
      end

      def update
        if @complex_subject.update(complex_subject_params)
          update_file_set
          message = 'Complex subject was successfully updated.'
          flash[:alert] = save_alert if params[:uploaded_files].present?
          redirect_to crc_dataset_complex_subject_path(@crc_dataset, @complex_subject), notice: message
        else
          render :edit
        end
      end

      def validate_subject_title
        folder_name = sanitize_title(params[:folder_name])
        subjects = @crc_dataset.complex_subjects
        .where(s3_folder_name: folder_name)
        .where.not(id: params[:current_object_id])
        invalid_length = false
        if folder_name.length < 3 or folder_name.length > 63
          invalid_length = true
          message = I18n.t('rdms.errors.complex_subject.title.length', value: folder_name)
        end
        message = I18n.t('rdms.errors.complex_subject.title.uniqueness', value: folder_name) if subjects.present?

        render json: { data_exist: (subjects.present? || invalid_length), error_message: message}
      end

      private

      def set_crc_dataset
        @crc_dataset = ::CrcDataset.find(params[:crc_dataset_id])
      end

      def set_complex_subject
        @complex_subject = ::ComplexSubject.find(params[:id])
      end

      def complex_subject_params
        params.require(:complex_subject).permit(:subject_title, :subject_species, :subject_type, :subject_sex, :subject_age)
      end

      def decide_layout
        layout = case action_name
                 when 'show'
                   '1_column'
                 else
                   'dashboard'
                 end
        File.join(theme, layout)
      end

      def update_file_set
        if params[:uploaded_files].present? and params[:uploaded_files].any?
          uploaded_files = Hyrax::UploadedFile.find(params[:uploaded_files])
          AttachFilesToWorkJob.perform_later(@crc_dataset, uploaded_files, {for_complex_identifier: @complex_subject.source_identifier, for_complex_type: 'ComplexSubject'})
        end
      end

      def save_alert
        'Your files are being processed by RDMS in the background. The metadata and access controls you specified are being applied. You may need to refresh this page to see these updates.'
      end

      def check_tombstone
        if @crc_dataset.is_tombstoned == true
          redirect_to main_app.polymorphic_path(@crc_dataset)
        end
      end
    end
  end
end
