# frozen_string_literal: true

module HyraxHelper
  include ::BlacklightHelper
  include Hyrax::BlacklightOverride
  include Hyrax::HyraxHelperBehavior

  def available_translations
    {
      'de' => 'Deutsch',
      'en' => 'English',
      'es' => 'Español',
      'fr' => 'Français',
      'it' => 'Italiano',
      'pt-BR' => 'Português do Brasil',
      'zh' => '中文'
    }.slice(RDMS::Application.config.i18n.available_locales )
  end

  def s3_file_download_url_for_file_set(file_set_id)
    s3 = S3StorageService.new
    s3.init_client

    file_set = FileSet.find(file_set_id)
    bucket_name = s3.sanitise_name(file_set.parent_works.first.id)

    s3.presigned_url_for_download(bucket_name, file_set.object_key_on_s3)
  end

  def can_create_work_in_group?(group_id)
    current_user.admin? || current_user.crc_manager_for?(group_id) || current_user.group_manager_for?(group_id) || current_user.group_member_for?(group_id)
  end

  def can_manage_managers_for_group?(presenter)
    current_user.admin? || current_user.crc_manager_for?(presenter.id)
  end

  def can_manage_members_for_group?(presenter)
    current_user.admin? || current_user.crc_manager_for?(presenter.id) || current_user.group_manager_for?(presenter.id)
  end

  def manager_role(presenter)
    Collection.find(presenter.id).manager_role
  end

  def member_role(presenter)
    Collection.find(presenter.id).member_role
  end

  def manager_role_for_group(presenter)
    Collection.find(presenter.id).manager_role_for_group
  end

  def member_role_for_group(presenter)
    Collection.find(presenter.id).member_role_for_group
  end

  def is_tombstonable?(presenter)
    presenter.tombstone_status != ['initiated'] && (is_published?(presenter) || is_archived?(presenter))
  end

  def is_published?(presenter)
    ['deposited', 'published'].include?(presenter.workflow.state)
  end

  def is_archived?(presenter)
    presenter.workflow.state == 'archived'
  end

  def tombstone_out_of_date?(presenter)
    ((Date.today) - presenter.tombstone_date.first.to_date).to_i >= ENV['NUMBER_OF_DAYS_TO_TOMBSTONE'].to_i
  end

  def is_tombston_initiated?(presenter)
    presenter.tombstone_status.present? && presenter.tombstone_status == ['initiated']
  end

  def is_permanently_tombstoned?(presenter)
    presenter.tombstone_status == ['deleted'] && presenter.is_tombstoned
  end

  def facet_fields_date_format(date)
    date.to_date.strftime('%Y-%m-%d')
  end

  def show_rerun_action?(importer)
    padding_imports(importer).zero? && 
    importer.last_run && 
    ((importer.last_run.failed_records.positive?) || 
    (importer.parser.total.positive? && importer.last_run.processed_records.zero? )) 
  end

  def show_marked_as_complete_action?(importer)
    padding_imports(importer).zero? && 
    importer.last_run && 
    importer.last_run.failed_records.zero? && 
    importer.last_run.enqueued_records.positive?
  end

  def padding_imports(importer)
    importer.entries.left_outer_joins(:latest_status).where('bulkrax_statuses.status_message IS NULL').count
  end

  def available_user_groups(ability:)
    return ::User.group_service.role_names if ability.admin?

    ability.user_groups - ['public']
  end


  def filter_metadata_json_file(object)
    metadata = object.to_json
    parsed_data = JSON.parse(metadata)

    case object.class.name
    when 'ComplexSubject'
      filtered_data = parsed_data.slice('id', 'subject_title', 'subject_species', 'subject_type', 'subject_sex', 'subject_age')
    when 'ComplexSession'
      filtered_data = parsed_data.slice('id', 'session_title', 'session_date_recorded')
    when 'ComplexModality'
      filtered_data = parsed_data.slice('id', 'modality_title', 'modality')
    else
      filtered_data = parsed_data.slice('id', 'depositor', 'date_created', 'date_modified',
                                        'title', 'description', 'resource_type', 'keyword', 'license',
                                        'alternative_title', 'subject', 'language', 'based_near', 'software_version',
                                        'publisher', 'doi', 'files_size', 'format',
                                        'experiment_description', 'crc_resource_type', 'coverage', 'approval_number',
                                        'extra_information')
      filtered_data['Creators and Contributors'] = parsed_data['complex_person'].map { |person| person.slice('first_name', 'last_name', 'name', 'email', 'role', 'orcid', 'affiliation')} if parsed_data['complex_person'].present?
      filtered_data['Related items'] = parsed_data['complex_relation'].map { |relation| relation.slice('title', 'url', 'relationship')} if parsed_data['complex_relation'].present?
      filtered_data['Date'] = parsed_data['complex_date'].map { |date| date.slice('date', 'description')} if parsed_data['complex_date'].present?
      filtered_data['Funding reference'] = parsed_data['complex_funding_reference'].map { |funding| funding.slice('funder_identifier', 'funder_name', 'award_number', 'award_uri', 'award_title')} if parsed_data['complex_funding_reference'].present?
      filtered_data['Identifiers'] = parsed_data['complex_identifier'].map { |identifier| identifier.slice('identifier', 'scheme')} if parsed_data['complex_identifier'].present?
      filtered_data
    end

    filtered_data.to_json
  end

  def download_all_button_available?(work_id)
    work = ActiveFedora::Base.find(work_id)
    s3 = S3StorageService.new
    s3.init_client
    bucket_name = s3.sanitise_name(work.id)
    meta_object = s3.system_metadata_object(bucket_name)

    work.date_modified < DateTime.parse(meta_object.last_modified.to_s)
  end

  def restricted_file_names?(file_name)
    parsed_file =  YAML.load_file(Rails.root.join('config', 'restricted_files.yml'))
    parsed_file['restricted_file_names'].map(&:downcase).include?(file_name.downcase)
  end

  def truncate_title(title)
    title.truncate(30)
  end

  # This method is used to get a Font Awesome class name or icon image path for different file types.
  # For example:
  #   file_icon_for('document.pdf') #=> "fa fa-file-pdf-o" OR "/pdf_icon.svg"
  def file_icon_for(file)
    extension = File.extname(file).downcase

    case extension
    when ".pdf"
      "fa fa-file-pdf-o"
    when ".doc", ".docx"
      "fa fa-file-word-o"
    when ".xls", ".xlsx", ".csv"
      "fa fa-file-excel-o"
    when ".ppt", ".pptx"
      "fa fa-file-powerpoint-o"
    when ".png", ".jpg", ".jpeg", ".gif", ".svg"
      "fa fa-file-image-o"
    when ".zip", ".rar", ".tar", ".gz"
      "fa fa-file-archive-o"
    when ".mp4", ".avi", ".mov", ".mkv", ".webm", ".wmv"
      "fa fa-file-video-o"
    when ".mp3", ".wav", ".ogg"
      "fa fa-file-audio-o"
    when ".txt", ".md", ".rtf", ".odt"
      "fa fa-file-text-o"
    when ".html", ".htm", ".xml"
      "fa fa-file-code-o"
    else
      "fa fa-file-o"
    end
  end

  def latest_varification_report_details(entry)
    # Define the file pattern and search path
    download_path = ENV.fetch('DOWNLOAD_PATH', '/shared/downloads')
    report_pattern = "analysis_report_#{entry.importerexporter_id}_#{entry.id}_*.csv"
    matching_files = Dir.glob(File.join(download_path, report_pattern))
  
    # Return nil if no matching files are found
    return [nil, nil, nil] if matching_files.empty?
  
    # Find the newest file based on modification time
    newest_file = matching_files.max_by { |file| File.mtime(file) }
  
    # Extract file details
    base_name = File.basename(newest_file, ".*")
    extension = File.extname(newest_file)
    date_time_str, status = base_name.split('_')[-2..]
  
    # Parse the date and time
    date, raw_time = date_time_str.split('T')
    formatted_time = raw_time.gsub('-', ':')
    parsed_datetime = Time.parse("#{date} #{formatted_time}")
  
    [status.titleize, parsed_datetime, newest_file]
  end

  def list_collections_to_display
    collections_to_display = []
    file_contents = File.new(Rails.root.join('config', 'collections_to_display.yml')).read
    template = ERB.new file_contents
    parsed_file = YAML.load(template.result(binding))
    parsed_file['collections_to_display'].each do |collection_name|
      collection = Collection.where(title: collection_name).first
      next unless can? :read, collection

      collections_to_display.append({id: collection.id, title: collection.title&.first, url: "/dashboard/collections/#{collection.id}"})
      # collections_to_display.append({id: c.id, title: c.title&.first, url: dashboard_collection_path(c.id)})
    end
    collections_to_display
  end
end
